@@ -1,4 +1,4 @@
-# $Id: Simple.pm,v 1.15 2003/06/13 10:27:20 grantm Exp $
+# $Id: Simple.pm,v 1.17 2004/02/29 09:48:17 grantm Exp $
 
 package XML::Simple;
 
@@ -53,7 +53,7 @@ use vars qw($VERSION @ISA @EXPORT @EXPORT_OK $PREFERRED_PARSER);
 @ISA               = qw(Exporter);
 @EXPORT            = qw(XMLin XMLout);
 @EXPORT_OK         = qw(xml_in xml_out);
-$VERSION           = '2.08';
+$VERSION           = '2.10';
 $PREFERRED_PARSER  = undef;
 
 my $StrictMode     = 0;
@@ -70,7 +70,7 @@ my @KnownOptIn     = qw(keyattr keeproot forcecontent contentkey noattr
 
 my @KnownOptOut    = qw(keyattr keeproot contentkey noattr
                         rootname xmldecl outputfile noescape suppressempty
-                        grouptags nsexpand handler noindent);
+                        grouptags nsexpand handler noindent attrindent nosort);
 
 my @DefKeyAttr     = qw(name key id);
 my $DefRootName    = qq(opt);
@@ -381,9 +381,7 @@ sub StorableRestore {
   return unless(-r $cachefile);
   return unless((stat($cachefile))[9] > (stat($filename))[9]);
 
-  unless($INC{'Storable.pm'}) {
-    require Storable;           # We didn't need it until now
-  }
+  require Storable;           # We didn't need it until now
   
   return(Storable::lock_retrieve($cachefile));
   
@@ -431,9 +429,7 @@ sub MemShareRestore {
 sub MemCopySave {
   my($data, $filename) = @_;
 
-  unless($INC{'Storable.pm'}) {
-    require Storable;           # We didn't need it until now
-  }
+  require Storable;           # We didn't need it until now
   
   $MemCopyCache{$filename} = [time(), Storable::dclone($data)];
 }
@@ -478,7 +474,7 @@ sub XMLout {
     $self = new XML::Simple();
   }
 
-
+  croak "XMLout() requires at least one argument" unless(@_);
   my $ref = shift;
 
   $self->handle_options('out', @_);
@@ -614,12 +610,10 @@ sub handle_options  {
 
   # Merge in options passed to constructor
 
-  if($self->{def_opt}) {
-    foreach (keys(%known_opt)) {
-      unless(exists($opt->{$_})) {
-        if(exists($self->{def_opt}->{$_})) {
-          $opt->{$_} = $self->{def_opt}->{$_};
-        }
+  foreach (keys(%known_opt)) {
+    unless(exists($opt->{$_})) {
+      if(exists($self->{def_opt}->{$_})) {
+        $opt->{$_} = $self->{def_opt}->{$_};
       }
     }
   }
@@ -824,9 +818,7 @@ sub find_xml_file  {
   # If user did not supply a search path, default to current directory
 
   if(!@search_path) {
-    if(-e $file) {
-      return($file);
-    }
+    return($file) if(-e $file);
     croak "File does not exist: $file";
   }
 
@@ -970,13 +962,11 @@ sub collapse {
 
   # Turn arrayrefs into hashrefs if key fields present
 
-  my $count = 0;
   if($self->{opt}->{keyattr}) {
     while(($key,$val) = each %$attr) {
       if(defined($val)  and  UNIVERSAL::isa($val, 'ARRAY')) {
         $attr->{$key} = $self->array_to_hash($key, $val);
       }
-      $count++;
     }
   }
 
@@ -999,6 +989,7 @@ sub collapse {
 
   # Fold hashes containing a single anonymous array up into just the array
 
+  my $count = scalar keys %$attr;
   if($count == 1 
      and  exists $attr->{anon}  
      and  UNIVERSAL::isa($attr->{anon}, 'ARRAY')
@@ -1253,7 +1244,7 @@ sub value_to_xml {
   # Unfold hash to array if possible
 
   if(UNIVERSAL::isa($ref, 'HASH')      # It is a hash
-     and %$ref                         # and it's not empty
+     and keys %$ref                    # and it's not empty
      and $self->{opt}->{keyattr}       # and folding is enabled
      and $indent                       # and its not the root element
   ) {
@@ -1344,15 +1335,22 @@ sub value_to_xml {
     }
 
     if(keys %$ref) {
-      while(($key, $value) = each(%$ref)) {
-	next if(substr($key, 0, 1) eq '-');
+      my $first_arg = 1;
+      foreach my $key ($self->sorted_keys($name, $ref)) {
+        my $value = $ref->{$key};
+        next if(substr($key, 0, 1) eq '-');
         if(!defined($value)) {
           unless(exists($self->{opt}->{suppressempty})
              and !defined($self->{opt}->{suppressempty})
           ) {
             carp 'Use of uninitialized value' if($^W);
           }
-          $value = {};
+          if($key eq $self->{opt}->{contentkey}) {
+            $text_content = '';
+          }
+          else {
+            $value = {};
+          }
         }
         if(ref($value)  or  $self->{opt}->{noattr}) {
           push @nested,
@@ -1364,7 +1362,10 @@ sub value_to_xml {
             $text_content = $value;
           }
           else {
+            push @result, "\n$indent " . ' ' x length($name)
+              if($self->{opt}->{attrindent}  and  !$first_arg);
             push @result, ' ', $key, '="', $value , '"';
+            $first_arg = 0;
           }
         }
       }
@@ -1432,6 +1433,42 @@ sub value_to_xml {
 }
 
 
+##############################################################################
+# Method: sorted_keys()
+#
+# Returns the keys of the referenced hash sorted into alphabetical order, but
+# with the 'key' key (as in KeyAttr) first, if there is one.
+#
+
+sub sorted_keys {
+  my($self, $name, $ref) = @_;
+
+  return keys %$ref if $self->{opt}->{nosort};
+
+  my %hash = %$ref;
+  my $keyattr = $self->{opt}->{keyattr};
+
+  my @key;
+
+  if(ref $keyattr eq 'HASH') {
+    if(exists $keyattr->{$name} and exists $hash{$keyattr->{$name}->[0]}) {
+      push @key, $keyattr->{$name}->[0];
+      delete $hash{$keyattr->{$name}->[0]};
+    }
+  }
+  elsif(ref $keyattr eq 'ARRAY') {
+    foreach (@{$keyattr}) {
+      if(exists $hash{$_}) {
+        push @key, $_;
+        delete $hash{$_};
+        last;
+      }
+    }
+  }
+
+  return(@key, sort keys %hash);
+}
+
 ##############################################################################
 # Method: escape_value()
 #
@@ -1471,7 +1508,8 @@ sub hash_to_array {
 
   my($key, $value);
 
-  foreach $key (keys(%$hashref)) {
+  my @keys = $self->{opt}->{nosort} ? keys %$hashref : sort keys %$hashref;
+  foreach $key (@keys) {
     $value = $hashref->{$key};
     return($hashref) unless(UNIVERSAL::isa($value, 'HASH'));
 
@@ -1743,8 +1781,8 @@ An IO::Handle object will be read to EOF and its contents parsed. eg:
 =head2 XMLout()
 
 Takes a data structure (generally a hashref) and returns an XML encoding of
-that structure.  If the resulting XML is parsed using C<XMLin()>, it will
-return a data structure equivalent to the original. 
+that structure.  If the resulting XML is parsed using C<XMLin()>, it should
+return a data structure equivalent to the original (see caveats below). 
 
 The C<XMLout()> function can also be used to output the XML as SAX events
 see the C<Handler> option and L<"SAX SUPPORT"> for more details).
@@ -1779,6 +1817,12 @@ approach).
 Note also that although you can nest hashes and arrays to arbitrary levels,
 circular data structures are not supported and will cause C<XMLout()> to die.
 
+If you wish to 'round-trip' arbitrary data structures from Perl to XML and back 
+to Perl, then you should probably disable array folding (using the KeyAttr
+option) both with C<XMLout()> and with C<XMLin()>.  If you still don't get the 
+expected results, you may prefer to use L<XML::Dumper> which is designed for
+exactly that purpose.
+
 Refer to L<"WHERE TO FROM HERE?"> if C<XMLout()> is too simple for your needs.
 
 
@@ -1810,29 +1854,43 @@ affects both C<XMLin> and C<XMLout>)
 
 =back
 
-Both C<XMLin()> and C<XMLout()> expect a single argument followed by a list of
-options.  An option takes the form of a 'name => value' pair.  The options
-listed below are marked with 'B<in>' if they are recognised by C<XMLin()> and
-'B<out>' if they are recognised by C<XMLout()>.
+The option name headings below have a trailing 'comment' - a hash followed by
+two pieces of metadata:
+
+=over 4
+
+=item *
+
+Options are marked with 'I<in>' if they are recognised by C<XMLin()> and
+'I<out>' if they are recognised by C<XMLout()>.
+
+=item *
 
 Each option is also flagged to indicate whether it is:
 
- 'important'   - don't use the module until you understand this
+ 'important'   - don't use the module until you understand this one
  'handy'       - you can skip this on the first time through
  'advanced'    - you can skip this on the second time through
- 'SAX only'    - don't worry about this unless you're using SAX
+ 'SAX only'    - don't worry about this unless you're using SAX (or
+                 alternatively if you need this, you also need SAX)
  'seldom used' - you'll probably never use this unless you were the
                  person that requested the feature
 
+=back
+
 The options are listed alphabetically:
 
 Note: option names are no longer case sensitive so you can use the mixed case
 versions shown here; all lower case as required by versions 2.03 and earlier;
 or you can add underscores between the words (eg: key_attr).
 
-=over 4
 
-=item Cache => [ cache scheme(s) ] (B<in>) (B<advanced>)
+=head2 AttrIndent => 1 I<# out - handy>
+
+When you are using C<XMLout()>, enable this option to have attributes printed
+one-per-line with sensible indentation rather than all on one line.
+
+=head2 Cache => [ cache schemes ] I<# in - advanced>
 
 Because loading the B<XML::Parser> module and parsing an XML file can consume a
 significant number of CPU cycles, it is often desirable to cache the output of
@@ -1882,7 +1940,7 @@ the time when it was last parsed.  If the file is stored on an NFS filesystem
 synchronised with the clock where your script is run, updates to the source XML
 file may appear to be ignored.
 
-=item ContentKey => 'keyname' (B<in+out>) (B<seldom used>)
+=head2 ContentKey => 'keyname' I<# in+out - seldom used>
 
 When text content is parsed to a hash value, this option let's you specify a
 name for the hash key to override the default 'content'.  So for example:
@@ -1929,7 +1987,7 @@ rather than this (without the '-'):
     }
   }
 
-=item DataHandler => code_ref (B<in>) (B<SAX only>)
+=head2 DataHandler => code_ref I<# in - SAX only>
 
 When you use an B<XML::Simple> object as a SAX handler, it will return a
 'simple tree' data structure in the same format as C<XMLin()> would return.  If
@@ -1939,7 +1997,7 @@ B<XML::Simple> object and a reference to the data tree.  The return value from
 the subroutine will be returned to the SAX driver.  (See L<"SAX SUPPORT"> for
 more details).
 
-=item ForceArray => 1 (B<in>) (B<IMPORTANT!>)
+=head2 ForceArray => 1 I<# in - important>
 
 This option should be set to '1' to force nested elements to be represented
 as arrays even when there is only one.  Eg, with ForceArray enabled, this
@@ -1973,7 +2031,7 @@ arrays and therefore will not be candidates for folding to a hash.  (Given that
 the default value of 'KeyAttr' enables array folding, the default value of this
 option should probably also have been enabled too - sorry).
 
-=item ForceArray => [ name(s) ] (B<in>) (B<IMPORTANT!>)
+=head2 ForceArray => [ names ] I<# in - important>
 
 This alternative (and preferred) form of the 'ForceArray' option allows you to
 specify a list of element names which should always be forced into an array
@@ -1986,7 +2044,7 @@ necessary to enclose it in an arrayref.  Eg:
 
   ForceArray => qr/_list$/
 
-=item ForceContent => 1 (B<in>) (B<seldom used>)
+=head2 ForceContent => 1 I<# in - seldom used>
 
 When C<XMLin()> parses elements which have text content as well as attributes,
 the text content must be represented as a hash value rather than a simple
@@ -2009,7 +2067,7 @@ instead of:
     'y' => { 'a' => 2, 'content' => 'text2' }
   }
 
-=item GroupTags => { grouping tag => grouped tag } (B<in+out>) (B<handy>)
+=head2 GroupTags => { grouping tag => grouped tag } I<# in+out - handy>
 
 You can use this option to eliminate extra levels of indirection in your Perl
 data structure.  For example this XML:
@@ -2049,7 +2107,7 @@ the grouped elements.  Beware though that this will occur in all places that
 the 'grouping tag' name occurs - you probably don't want to use the same name
 for elements as well as attributes.
 
-=item Handler => object_ref (B<out>) (B<SAX only>)
+=head2 Handler => object_ref I<# out - SAX only>
 
 Use the 'Handler' option to have C<XMLout()> generate SAX events rather than 
 returning a string of XML.  For more details see L<"SAX SUPPORT"> below.
@@ -2062,7 +2120,7 @@ the handler object, it will be in UTF8 form regardless of the encoding you
 supply.  A future implementation of this option may generate the events 
 directly.
 
-=item KeepRoot => 1 (B<in+out>) (B<handy>)
+=head2 KeepRoot => 1 I<# in+out - handy>
 
 In its attempt to return a data structure free of superfluous detail and
 unnecessary levels of indirection, C<XMLin()> normally discards the root
@@ -2079,7 +2137,7 @@ Similarly, setting the 'KeepRoot' option to '1' will tell C<XMLout()> that the
 data structure already contains a root element name and it is not necessary to
 add another.
 
-=item KeyAttr => [ list ] (B<in+out>) (B<IMPORTANT!>)
+=head2 KeyAttr => [ list ] I<# in+out - important>
 
 This option controls the 'array folding' feature which translates nested
 elements from an array to a hash.  It also controls the 'unfolding' of hashes
@@ -2135,7 +2193,7 @@ C<ForceArray> option.  Without 'ForceArray', a single nested element will be
 rolled up into a scalar rather than an array and therefore will not be folded
 (since only arrays get folded).
 
-=item KeyAttr => { list } (B<in+out>) (B<IMPORTANT!>)
+=head2 KeyAttr => { list } I<# in+out - important>
 
 This alternative (and preferred) method of specifiying the key attributes
 allows more fine grained control over which elements are folded and on which
@@ -2193,64 +2251,64 @@ A '-' prefix would produce this result:
 
 As described earlier, C<XMLout> will ignore hash keys starting with a '-'.
 
-=item NoAttr => 1 (B<in+out>) (B<handy>)
+=head2 NoAttr => 1 I<# in+out - handy>
 
 When used with C<XMLout()>, the generated XML will contain no attributes.
 All hash key/values will be represented as nested elements instead.
 
 When used with C<XMLin()>, any attributes in the XML will be ignored.
 
-=item NormaliseSpace => 0 | 1 | 2 (B<in>) (B<handy>)
-
-This option controls how whitespace in text content is handled.  Recognised
-values for the option are:
-
-=over 4
-
-=item 0
-
-the default behaviour is for whitespace to be passed through unaltered (except
-of course for the normalisation of whitespace in attribute values which is
-mandated by the XML recommendation)
-
-=item 1
-
-whitespace is normalised in any value used as a hash key (normalising means
-removing leading and trailing whitespace and collapsing sequences of whitespace
-characters to a single space)
-
-=item 2
-
-whitespace is normalised in all text content
-
-=back
-
-Note: you can spell this option with a 'z' if that is more natural for you.
-
-=item RootName => 'string' (B<out>) (B<handy>)
-
-By default, when C<XMLout()> generates XML, the root element will be named
-'opt'.  This option allows you to specify an alternative name.
-
-Specifying either undef or the empty string for the RootName option will
-produce XML with no root elements.  In most cases the resulting XML fragment
-will not be 'well formed' and therefore could not be read back in by C<XMLin()>.
-Nevertheless, the option has been found to be useful in certain circumstances.
-
-=item NoEscape => 1 (B<out>) (B<seldom used>)
+=head2 NoEscape => 1 I<# out - seldom used>
 
 By default, C<XMLout()> will translate the characters 'E<lt>', 'E<gt>', '&' and
 '"' to '&lt;', '&gt;', '&amp;' and '&quot' respectively.  Use this option to
 suppress escaping (presumably because you've already escaped the data in some
 more sophisticated manner).
 
-=item NoIndent => 1 (B<out>) (B<seldom used>)
+=head2 NoIndent => 1 I<# out - seldom used>
 
 Set this option to 1 to disable C<XMLout()>'s default 'pretty printing' mode.
 With this option enabled, the XML output will all be on one line (unless there
 are newlines in the data) - this may be easier for downstream processing.
 
-=item NSExpand => 1 (B<in+out>) (B<handy - SAX only>)
+=head2 NoSort => 1 I<# out - seldom used>
+
+Newer versions of XML::Simple sort elements and attributes alphabetically (*),
+by default.  Enable this option to suppress the sorting - possibly for
+backwards compatibility.
+
+* Actually, sorting is alphabetical but 'key' attribute or element names (as in
+'KeyAttr') sort first.  Also, when a hash of hashes is 'unfolded', the elements
+are sorted alphabetically by the value of the key field.
+
+=head2 NormaliseSpace => 0 | 1 | 2 I<# in - handy>
+
+This option controls how whitespace in text content is handled.  Recognised
+values for the option are:
+
+=over 4
+
+=item *
+
+0 = (default) whitespace is passed through unaltered (except of course for the
+normalisation of whitespace in attribute values which is mandated by the XML
+recommendation)
+
+=item *
+
+1 = whitespace is normalised in any value used as a hash key (normalising means
+removing leading and trailing whitespace and collapsing sequences of whitespace
+characters to a single space)
+
+=item *
+
+2 = whitespace is normalised in all text content
+
+=back
+
+Note: you can spell this option with a 'z' if that is more natural for you.
+
+=head2 NSExpand => 1 I<# in+out handy - SAX only>
 
 This option controls namespace expansion - the translation of element and
 attribute names of the form 'prefix:name' to '{uri}name'.  For example the
@@ -2272,24 +2330,33 @@ C<XMLout> will emit XML which is not well formed.
 I<Note: You must have the XML::NamespaceSupport module installed if you want
 C<XMLout()> to translate URIs back to prefixes>.
 
-=item OutputFile => <file specifier> (B<out>) (B<handy>)
+=head2 OutputFile => <file specifier> I<# out - handy>
 
 The default behaviour of C<XMLout()> is to return the XML as a string.  If you
 wish to write the XML to a file, simply supply the filename using the
 'OutputFile' option.  Alternatively, you can supply an IO handle object instead
 of a filename.
 
-=item ParserOpts => [ XML::Parser Options ] (B<in>) (B<don't use this>)
+=head2 ParserOpts => [ XML::Parser Options ] I<# in - don't use this>
 
 I<Note: This option is now officially deprecated.  If you find it useful, email
 the author with an example of what you use it for.  Do not use this option to
 set the ProtocolEncoding, that's just plain wrong - fix the XML>.
 
-Use this option to specify parameters that should be passed to the constructor
-of the underlying XML::Parser object (which of course assumes you're not using
-SAX).
+This option allows you to pass parameters to the constructor of the underlying
+XML::Parser object (which of course assumes you're not using SAX).
 
-=item SearchPath => [ list ] (B<in>) (B<handy>)
+=head2 RootName => 'string' I<# out - handy>
+
+By default, when C<XMLout()> generates XML, the root element will be named
+'opt'.  This option allows you to specify an alternative name.
+
+Specifying either undef or the empty string for the RootName option will
+produce XML with no root elements.  In most cases the resulting XML fragment
+will not be 'well formed' and therefore could not be read back in by C<XMLin()>.
+Nevertheless, the option has been found to be useful in certain circumstances.
+
+=head2 SearchPath => [ list ] I<# in - handy>
 
 If you pass C<XMLin()> a filename, but the filename include no directory
 component, you can use this option to specify which directories should be
@@ -2303,7 +2370,7 @@ If the first parameter to C<XMLin()> is undefined, the default SearchPath
 will contain only the directory in which the script itself is located.
 Otherwise the default SearchPath will be empty.  
 
-=item SuppressEmpty => 1 | '' | undef (B<in> + B<out>) (B<handy>)
+=head2 SuppressEmpty => 1 | '' | undef I<# in+out - handy>
 
 This option controls what C<XMLin()> should do with empty elements (no
 attributes and no content).  The default behaviour is to represent them as
@@ -2315,9 +2382,10 @@ easier to test for in your code than a hash with no keys.
 
 The option also controls what C<XMLout()> does with undefined values.
 Setting the option to undef causes undefined values to be output as
-empty elements (rather than empty attributes).
+empty elements (rather than empty attributes), it also suppresses the
+generation of warnings about undefined values.
 
-=item Variables => { name => value } (B<in>) (B<handy>)
+=head2 Variables => { name => value } I<# in - handy>
 
 This option allows variables in the XML to be expanded when the file is read.
 (there is no facility for putting the variable names back if you regenerate
@@ -2328,7 +2396,7 @@ value or in the text content of an element.  If 'name' matches a key in the
 supplied hashref, C<${name}> will be replaced with the corresponding value from
 the hashref.  If no matching key is found, the variable will not be replaced.
 
-=item VarAttr => 'attr_name' (B<in>) (B<handy>)
+=head2 VarAttr => 'attr_name' I<# in - handy>
 
 In addition to the variables defined using C<Variables>, this option allows
 variables to be defined in the XML.  A variable definition consists of an
@@ -2356,7 +2424,7 @@ produces the following data structure:
            }
   }
 
-=item XMLDecl => 1  or  XMLDecl => 'string'  (B<out>) (B<handy>)
+=head2 XMLDecl => 1  or  XMLDecl => 'string'  I<# out - handy>
 
 If you want the output from C<XMLout()> to start with the optional XML
 declaration, simply set the option to '1'.  The default XML declaration is:
@@ -2367,8 +2435,6 @@ If you want some other string (for example to declare an encoding value), set
 the value of this option to the complete string you require.
 
 
-=back
-
 =head1 OPTIONAL OO INTERFACE
 
 The procedural interface is both simple and convenient however there are a
@@ -2427,7 +2493,7 @@ the following common mistakes will be detected and treated as fatal errors
 =item *
 
 Failing to explicitly set the C<KeyAttr> option - if you can't be bothered
-reading about this option, turn it off with: KeyAttr => []
+reading about this option, turn it off with: KeyAttr => [ ]
 
 =item *
 
@@ -2794,7 +2860,7 @@ You don't need help converting between different encodings
 In a serious XML project, you'll probably outgrow these assumptions fairly
 quickly.  This section of the document used to offer some advice on chosing a
 more powerful option.  That advice has now grown into the 'Perl-XML FAQ'
-document which you can find at: L<http://www.perlxml.net/perl-xml-faq.dkb>
+document which you can find at: L<http://perl-xml.sourceforge.net/faq/>
 
 The advice in the FAQ boils down to a quick explanation of tree versus
 event based parsers and then recommends:
@@ -2807,10 +2873,6 @@ L<XML::Twig> and more standards based DOM implementations - preferably one with
 XPath support.
 
 
-=head1 STATUS
-
-This version (2.08) is the current stable version.
-
 =head1 SEE ALSO
 
 B<XML::Simple> requires either L<XML::Parser> or L<XML::SAX>.
@@ -2824,7 +2886,7 @@ distribution as: L<XML::Simple::FAQ>
 
 =head1 COPYRIGHT 
 
-Copyright 1999-2003 Grant McLean E<lt>grantm@cpan.orgE<gt>
+Copyright 1999-2004 Grant McLean E<lt>grantm@cpan.orgE<gt>
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself. 
